local UnitsBlock = {} 

UnitsBlock.__index = UnitsBlock

setmetatable(UnitsBlock, {
	__call = function (cls, ...)
		return cls.new(...)
	end,
})

function UnitsBlock.new(xmlDoc)
	local self = setmetatable({}, UnitsBlock)

	self.xmlDoc = xmlDoc
	self.content = { }
	self.ContentFrame = Apollo.LoadForm(self.xmlDoc, "ContentFrame", nil, self)
	self.ContentList = self.ContentFrame:FindChild("ContentList")


	self.bgColor = CColor.new(1,1,1,0.8)
	self.barColor = CColor.new(1,0,0,0.5)
	self.isEnabled = true
	self.anchorFromTop = true
	self.barSize = {
		Width = 300,
		Height = 25
	}

	return self
end

----------------------
-- SAVE AND RESTORE --
----------------------
function UnitsBlock:Load(saveData)
	if saveData.bgColor ~= nil then
		self:SetBGColor(CColor.new(saveData.bgColor[1], saveData.bgColor[2], saveData.bgColor[3], saveData.bgColor[4]))
	end

	if saveData.barColor ~= nil then
		self:SetBarColor(CColor.new(saveData.barColor[1], saveData.barColor[2], saveData.barColor[3], saveData.barColor[4]))
	end

	if saveData.isEnabled ~= nil then
		self:SetEnabled(saveData.isEnabled)
	end

	if saveData.barSize ~= nil then
		self.barSize = saveData.barSize
	end

	if saveData.Position ~= nil then
		self.ContentFrame:SetAnchorOffsets(saveData.Position[1], saveData.Position[2], saveData.Position[1] + self.barSize.Width, saveData.Position[2] + self.ContentFrame:GetHeight())
	end

	if saveData.anchorFromTop ~= nil then
		self.anchorFromTop = saveData.anchorFromTop
	end
end

function UnitsBlock:GetSaveData()
	local left, top, right, bottom = self.ContentFrame:GetAnchorOffsets()
	local saveData = {
		bgColor = { self.bgColor.r, self.bgColor.g, self.bgColor.b, self.bgColor.a },
		barColor = { self.barColor.r, self.barColor.g, self.barColor.b, self.barColor.a },
		isEnabled = self.isEnabled,
		Position = { left, top },
		anchorFromTop = self.anchorFromTop,
		barSize = self.barSize,
	}
	return saveData
end
-----------------------
-- /SAVE AND RESTORE --
-----------------------

--------------------
-- GUI MANAGEMENT --
--------------------
function UnitsBlock:SetName(name)
	self.ContentFrame:FindChild("MoveAnchor"):SetText(name)
end

function UnitsBlock:SetPosition(x, y)
	self.ContentFrame:SetAnchorPoints(x, y, x, y)
end

function UnitsBlock:ResetPosition()
	self.ContentFrame:SetAnchorOffsets(-150, -200, 150, 200)
	self:SetBarWidth(self.barSize.Width)
	if not self.anchorFromTop then
		self:AnchorFromTop(false, true)
	end
end

local function SortContentByHealth(a, b)
	return a:GetData().current < b:GetData().current
end

function UnitsBlock:SetEnabled(isEnabled)
	self.isEnabled = isEnabled
	if not isEnabled then
		for _, bar in pairs(self.content) do
			self.content[bar.id] = nil
			bar.Frame:Destroy()
			self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
		end
	end
end

function UnitsBlock:IsEnabled()
	return self.isEnabled
end

function UnitsBlock:SetMovable(isMovable)
	self.ContentFrame:FindChild("MoveAnchor"):Show(isMovable)
	self.ContentFrame:SetStyle("Picture", isMovable)
	self.ContentFrame:SetStyle("Moveable", isMovable)
end

function UnitsBlock:SetBarWidth(width)
	local left, top, right, bottom = self.ContentFrame:GetAnchorOffsets()
	right = left + width
	self.ContentFrame:SetAnchorOffsets(left, top, right, bottom)
	self.barSize.Width = width
end

function UnitsBlock:SetBarHeight(height)
	self.barSize.Height = height
	for _, bar in pairs(self.content) do
		bar:SetHeight(height)
	end
	self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
end

function UnitsBlock:ClearAll()
	for _, bar in pairs(self.content) do
		self.content[bar.id] = nil
		bar.Frame:Destroy()
	end
	self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
end

function UnitsBlock:SetBGColor(color)
	self.bgColor = color
	for _, bar in pairs(self.content) do
		bar:SetBGColor(color)
	end
end

function UnitsBlock:SetBarColor(color)
	self.barColor = color
	for _, bar in pairs(self.content) do
		bar:SetBarColor(color)
	end
end

function UnitsBlock:AnchorFromTop(anchorTop, force)
	if self.anchorFromTop ~= anchorTop or force then
		self.anchorFromTop = anchorTop
		local left, top, right, bottom = self.ContentFrame:GetAnchorOffsets()
		if anchorTop then
			self.ContentFrame:SetAnchorOffsets(left, top + self.ContentFrame:GetHeight(), right, top + self.ContentFrame:GetHeight() + self.ContentFrame:GetHeight())
		else
			self.ContentFrame:SetAnchorOffsets(left, top - self.ContentFrame:GetHeight(), right, top - self.ContentFrame:GetHeight() + self.ContentFrame:GetHeight())
		end
		self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
	end
end

function UnitsBlock:GetAnchorPoint()
	return self.anchorFromTop and Window.CodeEnumArrangeOrigin.LeftOrTop or Window.CodeEnumArrangeOrigin.RightOrBottom
end
---------------------
-- /GUI MANAGEMENT --
---------------------

----------------------
-- BLOCK MANAGEMENT --
----------------------
function UnitsBlock:ProcessUnits()
	if self.isEnabled then
		for _, bar in pairs(self.content) do
			self:ProcessUnit(bar)
		end
		self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
	end
end

function UnitsBlock:ProcessUnit(bar)
	local unit = bar.unit
	if unit:IsDead() or not unit:IsValid() then
		self.content[bar.id] = nil
		bar.Frame:Destroy()
		self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
	else
		local HealthCurrent = unit:GetHealth()
		local HealthMax = unit:GetMaxHealth()
		bar:SetHealth(HealthCurrent,HealthMax)
	end
end

function UnitsBlock:AddUnit(options)
	local unitid = options.unit:GetId()
	local UnitBar = self.content[unitid]
	if UnitBar == nil then
		UnitBar = self:CreateBar(options)
		self.content[unitid] = UnitBar
		UnitBar:SetHealth(options.HealthCurrent,options.HealthMax)
		self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
	else
		self.content[unitid] = nil
		UnitBar.Frame:Destroy()
		UnitBar = self:CreateBar(options)
		self.content[unitid] = UnitBar
		UnitBar:SetHealth(options.HealthCurrent,options.HealthMax)
		self.ContentList:ArrangeChildrenVert(self:GetAnchorPoint(), SortContentByHealth)
	end
end

function UnitsBlock:CreateBar(options)
	local bar = RaiderinoLibs.HealthBar.new(self.xmlDoc, self, options)
	bar:SetBGColor(self.bgColor)
	bar:SetBarColor(self.barColor)
	bar:SetHeight(self.barSize.Height)
	return bar
end
-----------------------
-- /BLOCK MANAGEMENT --
-----------------------

if _G["RaiderinoLibs"] == nil then
	_G["RaiderinoLibs"] = { }
end
_G["RaiderinoLibs"]["UnitsBlock"] = UnitsBlock